/*******************************************************************************
Copyright (C) Marvell International Ltd. and its affiliates

This software file (the "File") is owned and distributed by Marvell 
International Ltd. and/or its affiliates ("Marvell") under the following
alternative licensing terms.  Once you have made an election to distribute the
File under one of the following license alternatives, please (i) delete this
introductory statement regarding license alternatives, (ii) delete the two
license alternatives that you have not elected to use and (iii) preserve the
Marvell copyright notice above.

********************************************************************************
Marvell Commercial License Option

If you received this File from Marvell and you have entered into a commercial
license agreement (a "Commercial License") with Marvell, the File is licensed
to you under the terms of the applicable Commercial License.

********************************************************************************
Marvell GPL License Option

If you received this File from Marvell, you may opt to use, redistribute and/or 
modify this File in accordance with the terms and conditions of the General 
Public License Version 2, June 1991 (the "GPL License"), a copy of which is 
available along with the File in the license.txt file or by writing to the Free 
Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 or 
on the worldwide web at http://www.gnu.org/licenses/gpl.txt. 

THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE IMPLIED 
WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY 
DISCLAIMED.  The GPL License provides additional details about this warranty 
disclaimer.
********************************************************************************
Marvell BSD License Option

If you received this File from Marvell, you may opt to use, redistribute and/or 
modify this File under the following licensing terms. 
Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

    *   Redistributions of source code must retain the above copyright notice,
	    this list of conditions and the following disclaimer. 

    *   Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution. 

    *   Neither the name of Marvell nor the names of its contributors may be 
        used to endorse or promote products derived from this software without 
        specific prior written permission. 
    
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef __INCmvGppH
#define __INCmvGppH

#include "mvGppRegs.h"
#include "mvCtrlEnvLib.h"
#include "mvCtrlEnvAddrDec.h"

#define BUFFALO_SUPPORT_VIRTUALIZED_GPP

/* These macros describes the GPP type. Each of the GPPs pins can        	*/
/* be assigned to act as a general purpose input or output pin.             */
#define	MV_GPP_IN	0xFFFFFFFF 		/* GPP input   */
#define MV_GPP_OUT  0    			/* GPP output  */


/* These macros describes the GPP Out Enable. */
#define	MV_GPP_OUT_DIS	0xFFFFFFFF         	/* Out pin disabled*/
#define MV_GPP_OUT_EN   0    			 	/* Out pin enabled*/

/* These macros describes the GPP Out Blinking. */
/* When set and the corresponding bit in GPIO Data Out Enable Control 		*/
/* Register is enabled, the GPIO pin blinks every ~100 ms (a period of		*/
/* 2^24 TCLK clocks).														*/
#define	MV_GPP_OUT_BLINK	0xFFFFFFFF         	/* Out pin blinking*/
#define MV_GPP_OUT_STATIC   0    			 	/* Out pin static*/


/* These macros describes the GPP Polarity. */
/* When set to 1 GPIO Data In Register reflects the inverted value of the	*/
/* corresponding pin.														*/

#define	MV_GPP_IN_INVERT	0xFFFFFFFF         	/* Inverted value is got*/
#define MV_GPP_IN_ORIGIN    0    			 	/* original value is got*/

/* mvGppOutEnablle - Enable a GPP (OUT) pin */
MV_STATUS mvGppOutEnablle(MV_U32 group, MV_U32 mask, MV_U32 value);

/* mvGppBlinkEn - Set a GPP (IN) Pin list to blink every ~100ms */
MV_STATUS mvGppBlinkEn(MV_U32 group, MV_U32 mask, MV_U32 value);

/* mvGppPolaritySet - Set a GPP (IN) Pin list Polarity mode. */
MV_STATUS mvGppPolaritySet(MV_U32 group, MV_U32 mask, MV_U32 value);

/* mvGppValueGet - Get a GPP Pin list value.*/
MV_U32 mvGppValueGet(MV_U32 group, MV_U32 mask);

/* mvGppValueSet - Set a GPP Pin list value. */
MV_STATUS mvGppValueSet (MV_U32 group, MV_U32 mask, MV_U32 value);

 #define BUFFALO_LED_FLAG_ON		1
 #define BUFFALO_LED_FLAG_OFF		2
 #define BUFFALO_LED_FLAG_BLINK_ON	3

 #define BUFFALO_HDD_POWER_ON		1
 #define BUFFALO_HDD_POWER_OFF		2

 #define BUFFALO_USB_POWER_ON		1
 #define BUFFALO_USB_POWER_OFF		2

 #define BUFFALO_FAN_CONTROL_STOP	0
 #define BUFFALO_FAN_CONTROL_SLOW	1
 #define BUFFALO_FAN_CONTROL_FAST	2
 #define BUFFALO_FAN_CONTROL_FULL	3

#if defined BUFFALO_SUPPORT_VIRTUALIZED_GPP
 /* caution !                                                           *
  * if you use virtualized gpp, this routine contain "if else" command, *
  * so, you can't use BIT_xxxx definition for "switch case" command.    *
  * you can't use switch command, but can use "if xx else if xx else xx *
  * routine.                                                            *
  *   Y.H.                                                              */

 struct buffalo_gpp_config_st{
	char	hdd_power;
	char	hdd_power1;
	char	usb_power;
	char	usb_power2;
	char	usb_power3;
	char	usb_power4;
	char	led_info;
	char	led_info_group;	// 上位nibble：グループ， 下位nibble：優先度， -1：排他制御対象外
	char	led_power;
	char	led_power_group;
	char	led_alarm;
	char	led_alarm_group;
	char	led_func;
	char	led_func_group;
	char	led_eth;
	char	led_eth_group;
	char	led_reserve1;
	char	led_wlan;
	char	led_secure;
	char	led_aoss;
	char	fan_full;
	char	fan_lock;
	char	sw_init;
	char	sw_power;
	char	sw_raid;
	char	sw_auto_power;
	char	sw_func;
	char	sw_aoss;
	char	fan_low;
	char	fan_high;
 };

 extern int buffalo_board_id_to_gpp_config[];
 extern struct buffalo_gpp_config_st buffalo_gpp_config[];
 extern struct buffalo_gpp_config_st buffalo_gpp_config_atribute[];
 extern MV_U32 gBoardId;

 #define ATRIBUTE_ORIGIN	0
 #define ATRIBUTE_REVERSE	1

 //#define GPP_BIT(x)	{if(x != -1) (1 << x); else -1}
 #define GPP_BIT(x)	((1 << x) & 0xffffffff)

 #define VGPP_CONFIG_NUM	buffalo_board_id_to_gpp_config[gBoardId]
 #define VGPP_CONFIG		buffalo_gpp_config[VGPP_CONFIG_NUM]

 #define PIN_NUM_HDD_POWER	VGPP_CONFIG.hdd_power
 #define PIN_NUM_HDD_POWER1	VGPP_CONFIG.hdd_power1
 #define PIN_NUM_USB_POWER	VGPP_CONFIG.usb_power
 #define PIN_NUM_USB_POWER2	VGPP_CONFIG.usb_power2
 #define PIN_NUM_USB_POWER3	VGPP_CONFIG.usb_power3
 #define PIN_NUM_USB_POWER4	VGPP_CONFIG.usb_power4
 #define PIN_NUM_LED_INFO	VGPP_CONFIG.led_info
 #define LED_INFO_GRP		VGPP_CONFIG.led_info_group
 #define PIN_NUM_LED_POWER	VGPP_CONFIG.led_power
 #define LED_POWER_GRP		VGPP_CONFIG.led_power_group
 #define PIN_NUM_LED_ALARM	VGPP_CONFIG.led_alarm
 #define LED_ALARM_GRP		VGPP_CONFIG.led_alarm_group
 #define PIN_NUM_LED_FUNC	VGPP_CONFIG.led_func
 #define LED_FUNC_GRP		VGPP_CONFIG.led_func_group
 #define PIN_NUM_LED_ETH	VGPP_CONFIG.led_eth
 #define LED_ETH_GRP		VGPP_CONFIG.led_eth_group
 #define PIN_NUM_LED_RESERVE1	VGPP_CONFIG.led_reserve1
 #define PIN_NUM_LED_WLAN	VGPP_CONFIG.led_wlan
 #define PIN_NUM_LED_SECURE	VGPP_CONFIG.led_secure
 #define PIN_NUM_LED_AOSS	VGPP_CONFIG.led_aoss
 #define PIN_NUM_FAN_FULL	VGPP_CONFIG.fan_full
 #define PIN_NUM_FAN_LOCK	VGPP_CONFIG.fan_lock
 #define PIN_NUM_SW_INIT	VGPP_CONFIG.sw_init
 #define PIN_NUM_SW_POWER	VGPP_CONFIG.sw_power
 #define PIN_NUM_SW_RAID	VGPP_CONFIG.sw_raid
 #define PIN_NUM_SW_AUTO_POWER	VGPP_CONFIG.sw_auto_power
 #define PIN_NUM_SW_FUNC	VGPP_CONFIG.sw_func
 #define PIN_NUM_SW_AOSS	VGPP_CONFIG.sw_aoss
 #define PIN_NUM_FAN_LOW	VGPP_CONFIG.fan_low
 #define PIN_NUM_FAN_HIGH	VGPP_CONFIG.fan_high

 #define BIT_HDD_POWER		GPP_BIT(PIN_NUM_HDD_POWER)
 #define BIT_HDD_POWER1		GPP_BIT(PIN_NUM_HDD_POWER1)
 #define BIT_USB_POWER		GPP_BIT(PIN_NUM_USB_POWER)
 #define BIT_USB_POWER2		GPP_BIT(PIN_NUM_USB_POWER2)
 #define BIT_USB_POWER3		GPP_BIT(PIN_NUM_USB_POWER3)
 #define BIT_USB_POWER4		GPP_BIT(PIN_NUM_USB_POWER4)
 #define BIT_LED_INFO		GPP_BIT(PIN_NUM_LED_INFO)
 #define BIT_LED_POWER		GPP_BIT(PIN_NUM_LED_POWER)
 #define BIT_LED_ALARM		GPP_BIT(PIN_NUM_LED_ALARM)
 #define BIT_LED_FUNC		GPP_BIT(PIN_NUM_LED_FUNC)
 #define BIT_LED_RESERVE1	GPP_BIT(PIN_NUM_LED_RESERVE1)
 #define BIT_LED_WLAN		GPP_BIT(PIN_NUM_LED_WLAN)
 #define BIT_LED_SECURE		GPP_BIT(PIN_NUM_LED_SECURE)
 #define BIT_LED_AOSS		GPP_BIT(PIN_NUM_LED_AOSS)
 #define BIT_FAN_FULL		GPP_BIT(PIN_NUM_FAN_FULL)
 #define BIT_FAN_LOCK		GPP_BIT(PIN_NUM_FAN_LOCK)
 #define BIT_SW_INIT		GPP_BIT(PIN_NUM_SW_INIT)
 #define BIT_SW_POWER		GPP_BIT(PIN_NUM_SW_POWER)
 #define BIT_SW_RAID		GPP_BIT(PIN_NUM_SW_RAID)
 #define BIT_SW_AUTO_POWER	GPP_BIT(PIN_NUM_SW_AUTO_POWER)
 #define BIT_SW_FUNC		GPP_BIT(PIN_NUM_SW_FUNC)
 #define BIT_SW_AOSS		GPP_BIT(PIN_NUM_SW_AOSS)
 #define BIT_FAN_LOW		GPP_BIT(PIN_NUM_FAN_LOW)
 #define BIT_FAN_HIGH		GPP_BIT(PIN_NUM_FAN_HIGH)

 #define VGPP_ATRIBUTE_CONFIG_NUM	buffalo_board_id_to_gpp_config[gBoardId]
 #define VGPP_ATRIBUTE_CONFIG		buffalo_gpp_config_atribute[VGPP_ATRIBUTE_CONFIG_NUM]

 #define ATRIBUTE_HDD_POWER		VGPP_ATRIBUTE_CONFIG.hdd_power
 #define ATRIBUTE_HDD_POWER1		VGPP_ATRIBUTE_CONFIG.hdd_power1
 #define ATRIBUTE_USB_POWER		VGPP_ATRIBUTE_CONFIG.usb_power
 #define ATRIBUTE_USB_POWER2		VGPP_ATRIBUTE_CONFIG.usb_power2
 #define ATRIBUTE_USB_POWER3		VGPP_ATRIBUTE_CONFIG.usb_power3
 #define ATRIBUTE_USB_POWER4		VGPP_ATRIBUTE_CONFIG.usb_power4
 #define ATRIBUTE_LED_INFO		VGPP_ATRIBUTE_CONFIG.led_info
 #define ATRIBUTE_LED_POWER		VGPP_ATRIBUTE_CONFIG.led_power
 #define ATRIBUTE_LED_ALARM		VGPP_ATRIBUTE_CONFIG.led_alarm
 #define ATRIBUTE_LED_FUNC		VGPP_ATRIBUTE_CONFIG.led_func
 #define ATRIBUTE_LED_RESERVE1		VGPP_ATRIBUTE_CONFIG.led_reserve1
 #define ATRIBUTE_LED_WLAN		VGPP_ATRIBUTE_CONFIG.led_wlan
 #define ATRIBUTE_LED_SECURE		VGPP_ATRIBUTE_COFNIG.led_secure
 #define ATRIBUTE_LED_AOSS		VGPP_ATRIBUTE_CONFIG.led_aoss
 #define ATRIBUTE_FAN_FULL		VGPP_ATRIBUTE_CONFIG.fan_full
 #define ATRIBUTE_FAN_LOCK		VGPP_ATRIBUTE_CONFIG.fan_lock
 #define ATRIBUTE_SW_INIT		VGPP_ATRIBUTE_CONFIG.sw_init
 #define ATRIBUTE_SW_POWER		VGPP_ATRIBUTE_CONFIG.sw_power
 #define ATRIBUTE_SW_RAID		VGPP_ATRIBUTE_CONFIG.sw_raid
 #define ATRIBUTE_SW_AUTO_POWER		VGPP_ATRIBUTE_CONFIG.sw_auto_power
 #define ATRIBUTE_SW_FUNC		VGPP_ATRIBUTE_CONFIG.sw_func
 #define ATRIBUTE_SW_AOSS		VGPP_ATRIBUTE_CONFIG.sw_aoss
 #define ATRIBUTE_FAN_LOW		VGPP_ATRIBUTE_CONFIG.fan_low
 #define ATRIBUTE_FAN_HIGH		VGPP_ATRIBUTE_CONFIG.fan_high
#else
 #define BIT_HDD_POWER		1
 #define BIT_USB_POWER		9
 #define BIT_LED_INFO		3
 #define BIT_LED_POWER		0
 #define BIT_LED_ALARM		2
 #define BIT_FAN_FULL		14
 #define BIT_FAN_LOCK		6
 #define BIT_SW_INIT		7
 #define BIT_SW_POWER		8
 #define BIT_SW_RAID		
 #define BIT_SW_AUTO_POWER	10
#endif

MV_VOID buffalo_gpp_led_control(MV_U32, MV_U32);
MV_VOID buffalo_gpp_hdd_power_control(MV_U32);
MV_VOID buffalo_gpp_usb_power_control(MV_U32);

// convenient macros.
#define buffalo_gpp_power_led_on()		buffalo_gpp_led_control(BIT_LED_POWER, BUFFALO_LED_FLAG_ON)
#define buffalo_gpp_power_led_off()		buffalo_gpp_led_control(BIT_LED_POWER, BUFFALO_LED_FLAG_OFF)
#define buffalo_gpp_power_led_blink_on()	buffalo_gpp_led_control(BIT_LED_POWER, BUFFALO_LED_FLAG_BLINK_ON)

#define buffalo_gpp_info_led_on()		buffalo_gpp_led_control(BIT_LED_INFO, BUFFALO_LED_FLAG_ON)
#define buffalo_gpp_info_led_off()		buffalo_gpp_led_control(BIT_LED_INFO, BUFFALO_LED_FLAG_OFF)
#define buffalo_gpp_info_led_blink_on()	buffalo_gpp_led_control(BIT_LED_INFO, BUFFALO_LED_FLAG_BLINK_ON)

#define buffalo_gpp_alarm_led_on()		buffalo_gpp_led_control(BIT_LED_ALARM, BUFFALO_LED_FLAG_ON)
#define buffalo_gpp_alarm_led_off()		buffalo_gpp_led_control(BIT_LED_ALARM, BUFFALO_LED_FLAG_OFF)
#define buffalo_gpp_alarm_led_blink_on()	buffalo_gpp_led_control(BIT_LED_ALARM, BUFFALO_LED_FLAG_BLINK_ON)

#define buffalo_gpp_func_led_on()		buffalo_gpp_led_control(BIT_LED_FUNC, BUFFALO_LED_FLAG_ON)
#define buffalo_gpp_func_led_off()		buffalo_gpp_led_control(BIT_LED_FUNC, BUFFALO_LED_FLAG_OFF)
#define buffalo_gpp_func_led_blink_on()		buffalo_gpp_led_control(BIT_LED_FUNC, BUFFALO_LED_FLAG_BLINK_ON)

#define buffalo_gpp_wlan_led_on()		buffalo_gpp_led_control(BIT_LED_WLAN, BUFFALO_LED_FLAG_ON)
#define buffalo_gpp_wlan_led_off()		buffalo_gpp_led_control(BIT_LED_WLAN, BUFFALO_LED_FLAG_OFF)
#define buffalo_gpp_wlan_led_blink_on()		buffalo_gpp_led_control(BIT_LED_WLAN, BUFFALO_LED_FLAG_BLINK_ON)

#define buffalo_gpp_secure_led_on()		buffalo_gpp_led_control(BIT_LED_SECURE, BUFFALO_LED_FLAG_ON)
#define buffalo_gpp_secure_led_off()		buffalo_gpp_led_control(BIT_LED_SECURE, BUFFALO_LED_FLAG_OFF)
#define buffalo_gpp_secure_led_blink_on()	buffalo_gpp_led_control(BIT_LED_SECURE, BUFFALO_LED_FLAG_BLINK_ON)

#define buffalo_gpp_aoss_led_on()		buffalo_gpp_led_control(BIT_LED_AOSS, BUFFALO_LED_FLAG_ON)
#define buffalo_gpp_aoss_led_off()		buffalo_gpp_led_control(BIT_LED_AOSS, BUFFALO_LED_FLAG_OFF)
#define buffalo_gpp_oass_led_blink_on()		buffalo_gpp_led_control(BIT_LED_AOSS, BUFFALO_LED_FLAG_BLINK_ON)

#define buffalo_gpp_led_on(x)		buffalo_gpp_led_control(x, BUFFALO_LED_FLAG_ON)
#define buffalo_gpp_led_off(x)		buffalo_gpp_led_control(x, BUFFALO_LED_FLAG_OFF)
#define buffalo_gpp_led_blink(x)		buffalo_gpp_led_control(x, BUFFALO_LED_FLAG_BLINK_ON)

#define buffalo_gpp_hdd_power_on()		buffalo_gpp_hdd_power_control(BUFFALO_HDD_POWER_ON)
#define buffalo_gpp_hdd_power_off()		buffalo_gpp_hdd_power_control(BUFFALO_HDD_POWER_OFF)

#define buffalo_gpp_usb_power_on()		buffalo_gpp_usb_power_control(BUFFALO_USB_POWER_ON)
#define buffalo_gpp_usb_power_off()		buffalo_gpp_usb_power_control(BUFFALO_USB_POWER_OFF)

#define buffalo_gpp_sw_is_power_pushed()	(MV_REG_READ(GPP_DATA_IN_REG(0)) & BIT_SW_POWER)
#define buffalo_gpp_sw_is_init_pushed()	(MV_REG_READ(GPP_DATA_IN_REG(0)) & BIT_SW_INIT) 
#define buffalo_gpp_sw_is_auto_pushed()	(MV_REG_READ(GPP_DATA_IN_REG(0)) & BIT_SW_AUTO_POWER)
#define buffalo_gpp_sw_is_func_pushed()	(MV_REG_READ(GPP_DATA_IN_REG(0)) & BIT_SW_FUNC)
#define buffalo_gpp_sw_is_aoss_pushed()	(MV_REG_READ(GPP_DATA_IN_REG(0)) & BIT_SW_AOSS)
#define buffalo_gpp_sw_is_both_pushed()	(MV_REG_READ(GPP_DATA_IN_REG(0)) & (BIT_SW_POWER | BIT_SW_INIT))

#define buffalo_gpp_fan_control_stop()	buffalo_gpp_fan_control(BUFFALO_FAN_CONTROL_STOP)
#define buffalo_gpp_fan_control_slow()	buffalo_gpp_fan_control(BUFFALO_FAN_CONTROL_SLOW)
#define buffalo_gpp_fan_control_fast()	buffalo_gpp_fan_control(BUFFALO_FAN_CONTROL_FAST)
#define buffalo_gpp_fan_control_full()	buffalo_gpp_fan_control(BUFFALO_FAN_CONTROL_FULL)

#endif /* #ifndef __INCmvGppH */ 

