/*******************************************************************************
Copyright (C) Marvell International Ltd. and its affiliates

This software file (the "File") is owned and distributed by Marvell 
International Ltd. and/or its affiliates ("Marvell") under the following
alternative licensing terms.  Once you have made an election to distribute the
File under one of the following license alternatives, please (i) delete this
introductory statement regarding license alternatives, (ii) delete the two
license alternatives that you have not elected to use and (iii) preserve the
Marvell copyright notice above.

********************************************************************************
Marvell Commercial License Option

If you received this File from Marvell and you have entered into a commercial
license agreement (a "Commercial License") with Marvell, the File is licensed
to you under the terms of the applicable Commercial License.

********************************************************************************
Marvell GPL License Option

If you received this File from Marvell, you may opt to use, redistribute and/or 
modify this File in accordance with the terms and conditions of the General 
Public License Version 2, June 1991 (the "GPL License"), a copy of which is 
available along with the File in the license.txt file or by writing to the Free 
Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 or 
on the worldwide web at http://www.gnu.org/licenses/gpl.txt. 

THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE IMPLIED 
WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY 
DISCLAIMED.  The GPL License provides additional details about this warranty 
disclaimer.
********************************************************************************
Marvell BSD License Option

If you received this File from Marvell, you may opt to use, redistribute and/or 
modify this File under the following licensing terms. 
Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

    *   Redistributions of source code must retain the above copyright notice,
	    this list of conditions and the following disclaimer. 

    *   Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution. 

    *   Neither the name of Marvell nor the names of its contributors may be 
        used to endorse or promote products derived from this software without 
        specific prior written permission. 
    
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#include "mvUart.h"

/* global */
// non pay load commands.
MV_U8 BOOT_START[]={0x00, 0x02};

// payload 1 byte commands.
MV_U8 SW_UPS_INT[]={0x80, 0x36};
MV_U8 GET_BOARD_ID[]={0x80, 0x3f, 0x41};
MV_U8 HDD_POWER_CONTROL_W[]={0x01, 0x3b};
MV_U8 HDD_POWER_CONTROL_R[]={0x80, 0x3b};
MV_U8 FAN_CONTROL_W[]={0x01, 0x33};
MV_U8 FAN_CONTROL_R[]={0x80, 0x33};
MV_U8 FAN_GET_RPM[]={0x80, 0x38};
MV_U8 BZ_ON[]={0x01, 0x30};
MV_U8 USB_POWER_CONTROL_W[]={0x01, 0x40};
MV_U8 USB_POWER_CONTROL_R[]={0x80, 0x40};

// payload 2 bytes commands.
MV_U8 LED_CPU_MCON_W[]={0x02, 0x50};
MV_U8 LED_CPU_MCON_R[]={0x80, 0x50};
MV_U8 LED_ON_OFF_W[]={0x02, 0x51};
MV_U8 LED_ON_OFF_R[]={0x80, 0x51};
MV_U8 LED_BLINK_W[]={0x02, 0x52};
MV_U8 LED_BLINK_R[]={0x80, 0x52};


MV_U8 buffalo_serialport_enable=0; 


//#define BUFFALO_DEBUG

#if defined BUFFALO_DEBUG
 #define TRACE(x)	if(buffalo_serialport_enable) x
#else
 #define TRACE(x)
#endif

/* static variables */
static volatile MV_UART_PORT* uartBase[MV_UART_MAX_CHAN]; 


/*******************************************************************************
* mvUartInit - Init a uart port.
*
* DESCRIPTION:
*       This routine Initialize one of the uarts ports (channels).
*	It initialize the baudrate, stop bit,parity bit etc.
*
* INPUT:
*       port - uart port number.
*	baudDivisor - baud divisior to use for the uart port.
*
* OUTPUT:
*       None.
*
* RETURN:
*	None.
*
*******************************************************************************/
MV_VOID mvUartInit(MV_U32 port, MV_U32 baudDivisor, MV_UART_PORT* base)
{
	volatile MV_UART_PORT *pUartPort;

	uartBase[port] = pUartPort = (volatile MV_UART_PORT *)base;

	pUartPort->ier = 0x00;
	pUartPort->lcr = LCR_DIVL_EN;           /* Access baud rate */
	pUartPort->dll = baudDivisor & 0xff;    /* 9600 baud */
	pUartPort->dlm = (baudDivisor >> 8) & 0xff;
#if defined(CONFIG_BUFFALO_PLATFORM) && !defined(ORG_PCB)
	if (port == 0){
		// for serial console
		pUartPort->lcr = LCR_8N1;               /* 8 data, 1 stop, no parity */
	}else{
		// for micon control
		pUartPort->lcr = 0x1f ;				/* 8bit even-party, 2stop */
	}
#else
	pUartPort->lcr = LCR_8N1;               /* 8 data, 1 stop, no parity */
#endif

	/* Clear & enable FIFOs */
	pUartPort->fcr = FCR_FIFO_EN | FCR_RXSR | FCR_TXSR;
	return;
}

/*******************************************************************************
* mvUartInit - Init a uart port.
*
* DESCRIPTION:
*       This routine puts one charachetr on one of the uart ports.
*
* INPUT:
*       port - uart port number.
*	c - character.
*
* OUTPUT:
*       None.
*
* RETURN:
*	None.
*
*******************************************************************************/
MV_VOID	mvUartPutc(MV_U32 port, MV_U8 c)
{
	volatile MV_UART_PORT *pUartPort = uartBase[port];
	while ((pUartPort->lsr & LSR_THRE) == 0) ;
	pUartPort->thr = c;
	return;
}

/*******************************************************************************
* mvUartInit - Init a uart port.
*
* DESCRIPTION:
*       This routine gets one charachetr from one of the uart ports.
*
* INPUT:
*       port - uart port number.
*
* OUTPUT:
*       None.
*
* RETURN:
*	carachter from the uart port.
*
*******************************************************************************/
MV_U8	mvUartGetc(MV_U32 port)
{
	volatile MV_UART_PORT *pUartPort = uartBase[port];
	while ((pUartPort->lsr & LSR_DR) == 0) ;
	return (pUartPort->rbr);
}

MV_U8	mvUartGetcWithTimeOut(MV_U32 port)
{
	volatile MV_UART_PORT *pUartPort = uartBase[port];
	volatile MV_U32 timeout = (mvCpuPclkGet()/1000); //wait about 1 msec
	
	while ((pUartPort->lsr & LSR_DR) == 0){
		timeout--;
		if(timeout==0)
			return 0xff;
	}
	return (pUartPort->rbr);
}
/*******************************************************************************
* mvUartInit - Init a uart port.
*
* DESCRIPTION:
*       This routine heck if a charachter is ready to be read from one of the
*	the uart ports.
*
* INPUT:
*       port - uart port number.
*
* OUTPUT:
*       None.
*
* RETURN:
*	None.
*
*******************************************************************************/
MV_BOOL mvUartTstc(MV_U32 port)
{
	volatile MV_UART_PORT *pUartPort = uartBase[port];
	return ((pUartPort->lsr & LSR_DR) != 0);
}

/*******************************************************************************
*
* DESCRIPTION:
*
* micon communication
*
*
* INPUT:
*      
*
* OUTPUT:
*       None.
*
* RETURN:
*	None.
*
*******************************************************************************/
/*saito change*/
#if defined(CONFIG_BUFFALO_PLATFORM)

#if !defined(CONFIG_BUFFALO_LGL_PLATFORM)
void
micon_communication_tx_reset(void)
{
	int i=0;
	for(i=0; i<35; i++){
		mvUartPutc(1, 0xff);
	}
}

MV_U8
micon_communication_rx_reset(void)
{
	int retry_count=100;
	while((mvUartGetcWithTimeOut(1) != 0xff)){
		if(retry_count==0)
			return -1;
		retry_count--;
	}
	return 0;
}

MV_U8
micon_communication_reset(void)
{
	micon_communication_tx_reset();
	return micon_communication_rx_reset();
}

MV_VOID send_micon_packet(MV_U8 data_num,MV_U8 *data)
{
#if defined(ORG_PCB)
	return;
#else
	MV_U8 loop ;
	MV_U8 sum  = 0;
	for(loop = 0 ;loop < data_num ;loop++)
	{
//		TRACE(printf("%s > sending packet 0x%02x\n", __FUNCTION__, *data));
		mvUartPutc(1, *data);
		sum += *data ;
		*data ++;
   		mvOsDelay(200);
   	}
	sum = 0 - sum ;
//	TRACE(printf("%s > sending packet(check sum) 0x%02x\n", __FUNCTION__, sum));
//	TRACE(printf("%s > send packet finished.\n\n", __FUNCTION__));
	mvUartPutc(1, sum);
#endif
}

MV_U8 rcv_micon_packet_ll(MV_U8 *data, MV_U8 *cmd)
{
	MV_U8 loop;
	MV_U8 sum  = 0;
	MV_U8 tmp_d;
	MV_U8 data_num;
	volatile MV_UART_PORT *pUartPort = uartBase[1];

	data_num = mvUartGetc(1);

	if((data_num & 0x7f) > 32)
	{
		printf("Invalid Receive Data size :%d \n",data_num);
		pUartPort->fcr = FCR_FIFO_EN | FCR_RXSR | FCR_TXSR;
		return 0xff ;
	}
	sum += data_num ;
	tmp_d = mvUartGetcWithTimeOut(1) ;
	cmd[0] = tmp_d;
	sum += tmp_d;
//	TRACE(printf("%s > Command %x h:",__FUNCTION__, tmp_d));
	mvOsDelay(200);
	for(loop = 0 ;loop < (data_num & 0x7f) +1 ;loop++)
	{
			tmp_d = mvUartGetcWithTimeOut(1) ;
			sum += tmp_d;
			*data = tmp_d;
			data ++ ;
//			TRACE(printf(" %xh:",tmp_d));
   			mvOsDelay(200);
   	}
	if(sum != 0)
	{
		printf("Check Sum Error \n") ;
		return 0xff ;
	}
//	TRACE(printf("\n"));
	return 0 ;
}

MV_U8
rcv_micon_packet(MV_U8 *data)
{
	MV_U8 tmp;
	rcv_micon_packet_ll(data, &tmp);
}

/*
MV_VOID lcd_disp_chara(unsigned char disp_plane, MV_U8 *data)
{
	MV_U8 send_data[34];
	MV_U8 get_disp[]= {0x80,0x32};
	int loop;
	volatile MV_UART_PORT *pUartPort = uartBase[1];
	pUartPort->fcr = FCR_FIFO_EN | FCR_RXSR | FCR_TXSR;
	loop = 0 ;
	send_data[loop ++] = 32 ;
	if (disp_plane > 2)
	{
		return ;
	}
	send_data[loop ++] = 0x90 + disp_plane;
	for (loop = 2 ;loop < 34 ; loop++)
	{
		send_data[loop] = *data;
		data ++ ;
	}

	send_micon_packet(34,&send_data) ;
	rcv_micon_packet(&send_data) ;
	send_micon_packet(2,&get_disp) ;
	rcv_micon_packet(&send_data) ;
	send_data[2] =  send_data[0] | (0x20 << disp_plane) ;
	send_data[1] = 0x32 ; 
	send_data[0] = 0x01 ;
	send_micon_packet(3,&send_data) ;
	rcv_micon_packet(&send_data) ;
	send_data[0] = 0 ;
	send_data[1] = 0x25 +disp_plane ;
	send_micon_packet(2,&send_data) ;
	rcv_micon_packet(&send_data) ;
}
*/
MV_VOID peri_error(MV_U8 *data)
{
	MV_U8 send_data[34] ;
	int loop ;
	volatile MV_UART_PORT *pUartPort = uartBase[1];
	/* Clear & enable FIFOs */
	pUartPort->fcr = FCR_FIFO_EN | FCR_RXSR | FCR_TXSR;
	loop = 0 ;
	send_data[loop ++] = 32 ;
	send_data[loop ++] = 0x93 ; /*peri_error*/
	for (loop = 2 ;loop < 34 ; loop++)
	{
		send_data[loop] = *data;
		data ++ ;
	}
	send_micon_packet(34,&send_data) ;
	rcv_micon_packet(&send_data) ;
}

MV_VOID tftp_alarm(void)
{
/*	unsigned char disp_plane[]= "  TFTP MODE     System from Eth." ;*/
	unsigned char disp_plane[]=    		"TFTP MODE    E06Lost boot image " ;
	unsigned char disp_plane_ip[]= 		"This Machine IP                 " ;
	unsigned char disp_plane_sv_ip[]= 	"Server  IP                      " ;
 	MV_U8 send_data[34] ;
 	MV_U8 get_disp[]= {0x80,0x32} ;
 	MV_U8 LCD_5S[]=   {0x00,0x18} ;
 	MV_U8 BZ_05_03[]= {0x01,0x30,0x10} ;
 	MV_U8 LED_BLGT[]= {0x01,0x3a,0xff} ;
 	MV_U8 LED_ERR1[]= {0x02,0x50,0x04,0x0f} ;
 	MV_U8 LED_ERR2[]= {0x02,0x51,0x04,0x0f} ;
 	MV_U8 LED_ERR3[]= {0x02,0x54,0x00,0x06} ;
	MV_U8 BOOT_END[] = { 0,0x03} ;
	char *get_ip ;
 	int loop ;
	volatile MV_UART_PORT *pUartPort = uartBase[1];
	if (BUFFALO_BOARD_TS_HTGL_R5 == mvBoardIdGet())
	{
		pUartPort->fcr = FCR_FIFO_EN | FCR_RXSR | FCR_TXSR;
		 loop = 0 ;
		 send_data[loop ++] = 32 ;
		 send_data[loop ++] = 0x90 ;
		 for (loop = 2 ;loop < 34 ; loop++)
		 {
		 	send_data[loop] = disp_plane[loop - 2];
		 }
			send_micon_packet(34,&send_data) ;
			rcv_micon_packet(&send_data) ;


		 loop = 0 ;
		 send_data[loop ++] = 32 ;
		 send_data[loop ++] = 0x91 ;
		 get_ip = getenv("ipaddr") ;
		 for (loop = 2 ;loop < 34 ; loop++)
		 {
		 	send_data[loop] =  disp_plane_ip[loop - 2];
		 }
		 for (loop = 18 ;loop < 34 ; loop++)
		 {
		 	if(*get_ip ==0)
		 	{
				break ;
		 	}else
		 	{
		 	send_data[loop] =  *get_ip++;
		 	}
		 }
			send_micon_packet(34,&send_data) ;
			rcv_micon_packet(&send_data) ;

		 loop = 0 ;
		 send_data[loop ++] = 32 ;
		 send_data[loop ++] = 0x92 ;
		get_ip = getenv("serverip") ;
		 for (loop = 2 ;loop < 34 ; loop++)
		 {
		 	send_data[loop] =  disp_plane_sv_ip[loop - 2];
		 }
		 for (loop = 18 ;loop < 34 ; loop++)
		 {
		 	if(*get_ip ==0)
		 	{
				break ;
		 	}else
		 	{
		 	send_data[loop] =  *get_ip++;
		 	}
		 }
			send_micon_packet(34,&send_data) ;
			rcv_micon_packet(&send_data) ;
		 
		 
			send_micon_packet(2,&get_disp) ;
			rcv_micon_packet(&send_data) ;
			send_data[2] =  send_data[0] | 0xe0  ; /*disp0 disp1 disp2*/
			send_data[1] = 0x32 ; 
			send_data[0] = 0x01 ;
			send_micon_packet(3,&send_data) ;
			rcv_micon_packet(&send_data) ;
			send_data[0] = 0 ;
			send_data[1] = 0x25 ;
			send_micon_packet(2,&send_data) ;
			rcv_micon_packet(&send_data) ;
			send_micon_packet(3,&LED_BLGT) ;
			rcv_micon_packet(&send_data) ;
			send_micon_packet(4,&LED_ERR1) ;
			rcv_micon_packet(&send_data) ;
			send_micon_packet(4,&LED_ERR2) ;
			rcv_micon_packet(&send_data) ;
			send_micon_packet(2,&LCD_5S) ;
			rcv_micon_packet(&send_data) ;
			send_micon_packet(2,&BOOT_END) ;
			rcv_micon_packet(&send_data) ;
	}else
	{
			send_micon_packet(4,&LED_ERR3) ;
			rcv_micon_packet(&send_data) ;
	}
	
			send_micon_packet(3,&BZ_05_03) ;
			rcv_micon_packet(&send_data) ;
}
MV_VOID fl_ok(void)
{
#if !defined(ORG_PCB)
/*	mvUartPutc(1, 0);
	mvUartPutc(1, 2);
	mvUartPutc(1, 0xfe);*/
#endif
}

MV_VOID phy_ng(void)
{
	volatile MV_UART_PORT *pUartPort = uartBase[1];
	/* Clear & enable FIFOs */
	printf("<<%s>>\n",__FUNCTION__);
	if (BUFFALO_BOARD_TS_HTGL_R5 == mvBoardIdGet())
	{
		unsigned char disp_plane[]= "Chip Error   E21Ethernet Failure" ;
		peri_error(&disp_plane) ;
		if(is_em_mode() != 0x00 )
		{
			pUartPort->fcr = FCR_FIFO_EN | FCR_RXSR | FCR_TXSR;
			return ;
		}
	}else
	{
		MV_U8 data[] = { 0,0x28} ;
		send_micon_packet(2,&data) ;
	}
}

MV_VOID unmatch_password(void)
{
	printf("<<%s>>\n",__FUNCTION__);
	if (BUFFALO_BOARD_TS_HTGL_R5 == mvBoardIdGet())
	{
		unsigned char disp_plane[]= "Password     E06Password Unmatch" ;
		peri_error(&disp_plane) ;
	}
}

MV_VOID can_not_mount(void)
{
	printf("<<%s>>\n",__FUNCTION__);
	if (BUFFALO_BOARD_TS_HTGL_R5 == mvBoardIdGet())
	{
		unsigned char disp_plane[]= "HD ALL       E07All HD Not Found" ;
		peri_error(&disp_plane) ;
	}
	else
	{
#if !defined(ORG_PCB)
	mvUartPutc(1, 0);
	mvUartPutc(1, 0x29);
	mvUartPutc(1, 0xd7);
#endif
	}
}
MV_VOID system_not_found(void)
{
	printf("<<%s>>\n",__FUNCTION__);
#if !defined(ORG_PCB)
	mvUartPutc(1, 0);
	mvUartPutc(1, 0x2a);
	mvUartPutc(1, 0xd6);
#endif
}
MV_VOID system_bootend(void)
{
	MV_U8 rcv_data[34] ;
	MV_U8 data[] = { 0,0x03} ;
	MV_U8 fun_high_data[] = { 1,0x33,0x02} ;
	printf("<<%s>>\n",__FUNCTION__);
	send_micon_packet(2,&data) ;
	rcv_micon_packet(&rcv_data) ;
	send_micon_packet(3,&fun_high_data) ;
	rcv_micon_packet(&rcv_data) ;
}

MV_VOID stop_sound(void)
{
	MV_U8 data[] = { 1,0x30,00} ;
	printf("<<%s>>\n",__FUNCTION__);
	send_micon_packet(3,&data) ;
}

MV_VOID system_shutdown(void)
{
	MV_U8 data1[] = { 0,0x0c} ;
	MV_U8 data2[] = { 0,0x06} ;
	printf("<<%s>>\n",__FUNCTION__);
	send_micon_packet(2,&data1) ;
  	mvOsDelay(200);
	send_micon_packet(2,&data2) ;
}
MV_U8 is_em_mode(void)
{
#if !defined(ORG_PCB)
	volatile MV_UART_PORT *pUartPort = uartBase[1];
	/* Clear & enable FIFOs */
	MV_U8 ss;
	pUartPort->fcr = FCR_FIFO_EN | FCR_RXSR | FCR_TXSR;
	mvUartPutc(1, 0x80);
	mvUartPutc(1, 0x3e);
	mvUartPutc(1, 0x42);

	ss = mvUartGetc(1) ;
	ss = mvUartGetc(1) ;
	ss = mvUartGetc(1) ;
	mvUartGetc(1) ;
	return ss ;
#else
	return 0;
#endif
}

MV_U32
mcon_get_borad_id(void)
{
#if !defined(ORG_PCB)
	volatile MV_UART_PORT *pUartPort = uartBase[1];
	int clock_divisor2 = (CFG_TCLK / 16)/ 38400 ;
 	mvUartInit(1, clock_divisor2, mvUartBase(1));
	/* Clear & enable FIFOs */
	MV_U8 ss;
	MV_U32 tt;
	pUartPort->fcr = FCR_FIFO_EN | FCR_RXSR | FCR_TXSR;
	mvOsDelay(200);
#if 0
	mvUartPutc(1, 0x80);
	mvUartPutc(1, 0x3f);
	mvUartPutc(1, 0x41);
#else
	//micon_communication_reset();
	send_micon_packet(sizeof(GET_BOARD_ID),&GET_BOARD_ID);
#endif
	mvOsDelay(200);
	ss = mvUartGetc(1) ;
	ss = mvUartGetc(1) ;
	ss = mvUartGetc(1) ;
	tt = (MV_U32)ss ;
	mvUartGetc(1) ;
	return  tt;
#else
	return 0;
#endif
}

MV_U32
mcon_common_communication_routine(size_t cmd_length, MV_U8 *send_cmd)
{
	MV_U8 rcv_data[34];
	MV_U8 rcv_cmd;
	int counter=0;

	memset(rcv_data, (char)NULL, sizeof(rcv_data));

	if(micon_communication_reset() == 0){
		//TRACE(printf("micon_communication reset success!\n"));
	}else{
		// rx buffer not empty proberbility is exist!.
		TRACE(printf("micon communication reset fialed!\n"));
	}

	while(counter < 3){
		send_micon_packet(cmd_length,send_cmd);
		rcv_micon_packet_ll(&rcv_data, &rcv_cmd);
		counter++;
		if(send_cmd[1] == rcv_cmd){
			break;
		}else{
			printf("command(0x%02x) result : unexpected command received(0x%02x). try again ...\n", send_cmd[1], rcv_cmd);
		}
	}
	return ((rcv_data[3] << 24) | (rcv_data[2] << 16) | 
		(rcv_data[1] << 8 ) | (rcv_data[0] << 0 ));
}

MV_U32
mcon_common_command_routine(MV_U8 rwflag, MV_U8 taskflag, MV_U32 control_bit)
{
	MV_U8 send_cmd[3];
	MV_U8 send_length=0;

	switch(rwflag){
	case MCON_WRITE:
		switch(taskflag){
		case MCON_HDD_CONTROL:
			TRACE(printf("<<WRITE:MCON_HDD_CONTROL>>\n"));
			send_cmd[0]=HDD_POWER_CONTROL_W[0];
			send_cmd[1]=HDD_POWER_CONTROL_W[1];
			send_length=3;
			break;
		case MCON_FAN_CONTROL:
			TRACE(printf("<<WRITE:MCON_FAN_CONTROL>>\n"));
			send_cmd[0]=FAN_CONTROL_W[0];
			send_cmd[1]=FAN_CONTROL_W[1];
			send_length=3;
			break;
		case MCON_LED_CPU_MCON:
			TRACE(printf("<<WRITE::MCON_LED_CPU_MCON>>\n"));
			send_cmd[0]=LED_CPU_MCON_W[0];
			send_cmd[1]=LED_CPU_MCON_W[1];
			send_length=4;
			break;
		case MCON_LED_ON_OFF:
			TRACE(printf("<<WRITE:MCON_LED_ON_OFF>>\n"));
			send_cmd[0] = LED_ON_OFF_W[0];
			send_cmd[1] = LED_ON_OFF_W[1];
			send_length=4;
			break;
		case MCON_LED_BLINK:
			TRACE(printf("<<WRITE:MCON_LED_BLINK>>\n"));
			send_cmd[0] = LED_BLINK_W[0];
			send_cmd[1] = LED_BLINK_W[1];
			send_length=4;
			break;
		case MCON_BZ_ON:
			TRACE(printf("<<WRITE:MCON_BZ_ON>>\n"));
			send_cmd[0] = BZ_ON[0];
			send_cmd[1] = BZ_ON[1];
			send_length=3;
			break;
		case MCON_USB_POWER_CONTROL:
			TRACE(printf("<<WRITE:MCON_USB_POWER_CONTROL>>\n"));
			send_cmd[0] = USB_POWER_CONTROL_W[0];
			send_cmd[1] = USB_POWER_CONTROL_R[1];
			send_length=3;
			break;
		default:
			break;
		}
		send_cmd[2]=(MV_U8)((control_bit & 0x000000ff) >> 0);
		send_cmd[3]=(MV_U8)((control_bit & 0x0000ff00) >> 8);
		send_cmd[4]=(MV_U8)((control_bit & 0x00ff0000) >> 16);
		send_cmd[5]=(MV_U8)((control_bit & 0xff000000) >> 24);
		return mcon_common_communication_routine(send_length, &send_cmd);
		break;
	case MCON_READ:
		switch(taskflag){
		case MCON_HDD_CONTROL:
			TRACE(printf("<<READ:MCON_HDD_CONTROL>>\n"));
			return mcon_common_communication_routine(sizeof(HDD_POWER_CONTROL_R), &HDD_POWER_CONTROL_R);
			break;
		case MCON_FAN_CONTROL:
			TRACE(printf("<<READ:MCON_FAN_CONTROL>>\n"));
			return mcon_common_communication_routine(sizeof(FAN_CONTROL_R), &FAN_CONTROL_R);
			break;
		case MCON_FAN_GET_RPM:
			TRACE(printf("<<READ:MCON_FAN_GET_RPM>>\n"));
			return mcon_common_communication_routine(sizeof(FAN_GET_RPM), &FAN_GET_RPM);
			break;
		case MCON_LED_CPU_MCON:
			TRACE(printf("<<READ:MCON_LED_CPU_MCON>>\n"));
			return mcon_common_communication_routine(sizeof(LED_CPU_MCON_R), &LED_CPU_MCON_R);
			break;
		case MCON_LED_ON_OFF:
			TRACE(printf("<<READ:MCON_LED_ON_OFF>>\n"));
			return mcon_common_communication_routine(sizeof(LED_ON_OFF_R), &LED_ON_OFF_R);
			break;
		case MCON_LED_BLINK:
			TRACE(printf("<<READ:MCON_LED_BLINK>>\n"));
			return mcon_common_communication_routine(sizeof(LED_BLINK_R), &LED_BLINK_R);
			break;
		case MCON_BOOT_START:
			TRACE(printf("<<READ:MCON_BOOT_START>>\n"));
			return mcon_common_communication_routine(sizeof(BOOT_START), &BOOT_START);
			break;
		case MCON_GET_SWITCH_STATUS:
			TRACE(printf("<<READ:MCON_GET_SWITCH_STATSU\n"));
			return mcon_common_communication_routine(sizeof(SW_UPS_INT), &SW_UPS_INT);
			break;
		case MCON_USB_POWER_CONTROL:
			TRACE(printf("<<READ:MCON_USB_POWER_CONTROL\n"));
			return mcon_common_communication_routine(sizeof(USB_POWER_CONTROL_R), &USB_POWER_CONTROL_R);
			break;
		default:	
			break;
		}
		break;
	default:
		break;
	}
}

MV_VOID
buffalo_software_power_off(void)
{
	system_bootend();
	mcon_hdd_power_all_off();
	mcon_led_all_off();
	mcon_usb_off();
	mcon_fan_stop();
}

MV_VOID
buffalo_software_power_on(void)
{
	mcon_hdd_power_all_on();
	mcon_led_all_mcon();
	mcon_fan_stop();
	mcon_usb_on();
	mcon_led_blink_start();
}
#endif // end of !CONFIG_BUFFALO_LGL_PLATFORM

#endif

